unit UMainSys;

interface

uses
	Windows, USmDx3D, DDUtils, DDVer, USmUtil, DirectDraw;

const
	MAX_IMAGE     = 3;
	MAX_OBJECT    = 200;
	PARTICLE_STEP = 11;

type
	TGameOption = record
		Handle        : THandle;
		Width         : Integer;
		Height        : Integer;
		Depth         : Integer;
		UseFullScreen : Boolean;
	end;

	TBaseObj = class
		constructor Create; virtual; abstract;

	private
		m_x, m_y   : Integer;

	public
		function  MainProc : Boolean; virtual; abstract;

	end;

	TParticle = class( TBaseObj )

		constructor Create( x, y : Integer; vx, vy, ax, ay : Double );

	private
		m_sx, m_sy : Double;
		m_vx, m_vy : Double;
		m_ax, m_ay : Double;

	public
		function  MainProc : Boolean; override;

	end;

	TPlane = class( TBaseObj )

		constructor Create( x, y : Integer );

	private
		m_x_charge   : Integer;

	public
		RemainShield : Integer;
		function  MainProc : Boolean; override;

	end;

	TMainSystem = class
		constructor Create( Handle : THandle );
		procedure   Free;

	private
		hFontTitle  : HFONT;
		hFontNormal : HFONT;

	public
		CommandMessage : ( cmStartTitle, cmTitle, cmStartGame, cmGame, cmStartEnding, cmEnding );
		BMPImage   : array[0..Pred( MAX_IMAGE   )] of II_DirectDrawSurface;

		function  MainProcTitle  : Boolean;
		function  MainProcGame   : Boolean;
		function  MainProcEnding : Boolean;

		function  MainProc : Boolean;

	end;

var
	MainSystem  : TMainSystem;
	SmDirect3D  : TSmDirect3D;
	GameOption  : TGameOption;
	GameCursor  : TPoint;
	GameKey     : Word;
	ResourceDLL : integer = 0;

implementation

var
	Objects     : array[0..Pred( MAX_OBJECT )] of TBaseObj;
	ColorMode   : Integer;

{ TParticle }

constructor TParticle.Create( x, y : Integer; vx, vy, ax, ay : Double );
begin
	m_x  := x;
	m_y  := y;
	m_sx := x;
	m_sy := y;
	m_vx := vx;
	m_vy := vy;
	m_ax := ax;
	m_ay := ay;
end;

function    TParticle.MainProc : Boolean;
const
{
	SourR = 31;
	SourG = 20;
	SourB = 8;
}
	SourG = 31;
	SourB = 20;
	SourR = 8;
var
	ARect  : TRect;
	x, y   : Integer;
	pSour  : PWORD;
	pDest  : PWORD;

	SourA  : WORD;
	DestA  : WORD;
	DestR  : WORD;
	DestG  : WORD;
	DestB  : WORD;
	ShiftR : WORD;
	ShiftG : WORD;
	ShiftB : WORD;
begin
	m_vx := m_vx + m_ax;
	m_vy := m_vy + m_ay;
	m_sx := m_sx + m_vx;
	m_sy := m_sy + m_vy;
	m_x  := Round( m_sx );
	m_y  := Round( m_sy );

	if ( ColorMode = 16 ) then begin
		ShiftR := 11;
		ShiftG :=  6;
		ShiftB :=  0;
	end
	else begin
		ShiftR := 10;
		ShiftG :=  5;
		ShiftB :=  0;
	end;

	if ( m_x < 8 ) or ( m_x >= GameOption.Width-8 ) or ( m_y < 8 ) or ( m_y >= GameOption.Height-8 ) then begin
		Result := FALSE;

	end else begin
		ARect := Bounds( 0, 14, 4, 4 );
		SmDirect3D.DirectDrawSurface.BltFast( m_x, m_y, MainSystem.BMPImage[0], @ARect, 1 );
		Result := TRUE;
	end;
end;

{ TPlane }

constructor TPlane.Create( x, y : Integer );
begin
	m_x := x;
	m_y := y;

	m_x_charge   := 0;
	RemainShield := 300 * 5;
end;

function    TPlane.MainProc : Boolean;

	function Sign( i : Integer ) : Integer;
	begin
		if      i = 0 then Result := 0
		else if i > 0 then Result := 1
		else               Result := -1
	end;
const
	PLANE_XSIZE = 14;
	PLANE_YSIZE = 13;
	PLANE_XGAP  = 15;
var
	ARect  : TRect;
	i      : Integer;
	Charge : Integer;
	Temp   : TParticle;
	Dist   : TPoint;
begin
	Charge := 0;

	// 񵿱 Է
	if HIBYTE( GetAsyncKeyState( VK_LEFT  ) ) > 0 then begin
		Dec( m_x ); Charge := -1;
	end;
	if HIBYTE( GetAsyncKeyState( VK_RIGHT ) ) > 0 then begin
		Inc( m_x ); Charge :=  1;
	end;
	if HIBYTE( GetAsyncKeyState( VK_UP    ) ) > 0 then
		Dec( m_y );
	if HIBYTE( GetAsyncKeyState( VK_DOWN  ) ) > 0 then
		Inc( m_y );

	// ߷
	if ( HIBYTE( GetAsyncKeyState( 32 ) ) > 0 ) and ( RemainShield > 0 ) then begin
		Dec( RemainShield );
		for i := 1 to Pred( MAX_OBJECT ) do begin
			if Assigned( Objects[i] ) then begin
				Dist.x := Objects[i].m_x - m_x;
				Dist.y := Objects[i].m_y - m_y;
				if ( Dist.x*Dist.x + Dist.y*Dist.y ) < 2500 then begin
					Temp  := TParticle( Objects[i] );
					Temp.m_ax := Sign(Dist.x) * (50-Abs(Dist.x)) * 0.001;
					Temp.m_ay := Sign(Dist.y) * (50-Abs(Dist.y)) * 0.001;
				end;
			end;
		end;
	end;

	// 
	if ( m_x < 0 ) then m_x := 0;
	if ( m_y < 0 ) then m_y := 0;
	if ( m_x >= GameOption.Width  - PLANE_XSIZE ) then m_x := Pred( GameOption.Width  - PLANE_XSIZE );
	if ( m_y >= GameOption.Height - PLANE_YSIZE ) then m_y := Pred( GameOption.Height - PLANE_YSIZE );

	//  ̱
	if ( Charge > 0 ) then begin
		if ( m_x_charge > 0 ) then
			Inc( m_x_charge )
		else
			m_x_charge := 1;
	end
	else if ( Charge < 0 ) then begin
		if ( m_x_charge < 0 ) then
			Dec( m_x_charge )
		else
			m_x_charge := -1;
	end
	else begin
		m_x_charge := 0;
	end;

	if      m_x_charge =  0 then
		i     := 2
	else if m_x_charge < -9 then
		i     := 0
	else if m_x_charge <  0 then
		i     := 1
	else if m_x_charge >  9 then
		i     := 4
	else if m_x_charge >  0 then
		i     := 3;

	ARect  := Bounds( PLANE_XGAP*i, 0, PLANE_XSIZE, PLANE_YSIZE );
	SmDirect3D.DirectDrawSurface.BltFast( m_x, m_y, MainSystem.BMPImage[0], @ARect, 1 );

	// 浹 üũ
	ARect  := Bounds( m_x, m_y, PLANE_XSIZE, PLANE_YSIZE );
	for i := 1 to Pred( MAX_OBJECT ) do begin
		if Assigned( Objects[i] ) then begin
			if PtInRect( ARect, Point( Objects[i].m_x, Objects[i].m_y ) ) then begin
				Result := FALSE;
				Exit;
			end;
		end;
	end;

	Result := TRUE;
end;

{ TMainSystem }

constructor TMainSystem.Create( Handle : THandle );
var
	hBM           : HBITMAP;
	TempSize      : TPoint;
	DDPixelFormat : TDDPixelFormat;
	ColorKey      : TDDColorKey;
	LogFont       : TLOGFONT;
begin
	Inherited Create;

	// Ķ ʱȭ
	ZeroMemory( @BMPImage, sizeof( BMPImage  ) );
	ZeroMemory( @Objects,  sizeof( Objects ) );

	GameOption.Handle := Handle;

	// Direct X ʱȭ
	with GameOption do
		SmDirect3D    := TSmDirect3D.Create3D( Handle, Width, Height, Depth, UseFullScreen );

	// Image ε
	BMPImage[0] := DDCreateSurface( SmDirect3D.DirectDraw, GameOption.Width, GameOption.Height, TRUE );

	hBM := LoadImage( 0, PChar( 'Totkun.BMP' ), IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE or LR_CREATEDIBSECTION );
	if hBM <> 0 then begin
		DDCopyBitmapEx( BMPImage[0], 0, 0, hBM, 0, 0, 0, 0 );
		DeleteObject( hBM );
	end;

	ColorKey.dwColorSpaceLowValue  := $001F;
	ColorKey.dwColorSpaceHighValue := ColorKey.dwColorSpaceLowValue;
	BMPImage[0].SetColorKey( DDCKEY_SRCBLT, @ColorKey );

	ZeroMemory( @DDPixelFormat, sizeof( DDPixelFormat ) );
	DDPixelFormat.dwSize := sizeof( DDPixelFormat );
	BMPImage[0].GetPixelFormat( DDPixelFormat );

	if DDPixelFormat.dwRBitMask = $0000F800 then
		ColorMode := 16
	else
	if DDPixelFormat.dwRBitMask = $00007C00 then
		ColorMode := 15
	else
		ColorMode := 16;

	begin
		ZeroMemory( @LogFont, sizeof( LogFont ) );
		LogFont.lfWeight   := FW_BOLD;
		LogFont.lfHeight   := 64;
		LogFont.lfCharSet  := ANSI_CHARSET;
		LogFont.lfQuality  := PROOF_QUALITY;
		LogFont.lfFaceName := '';
		hFontTitle         := CreateFontIndirect( LogFont );

		LogFont.lfWeight   := FW_DONTCARE;
		LogFont.lfHeight   := 24;
		LogFont.lfFaceName := '';
		hFontNormal        := CreateFontIndirect( LogFont );
	end;

	CommandMessage := cmStartTitle;
end;

procedure   TMainSystem.Free;
var
	i : Integer;
begin
	DeleteObject( hFontTitle  );
	DeleteObject( hFontNormal );

	for i := 0 to Pred( MAX_IMAGE ) do begin
		if Assigned( BMPImage[i] ) then
			BMPImage[i] := nil;
	end;

	SmDirect3D.Free;

	Inherited;
end;

var
	StartTick : Integer;

function    TMainSystem.MainProc : Boolean;
var
	i : Integer;
begin
	case CommandMessage of
		cmStartTitle  :
		begin
			GetAsyncKeyState( VK_RETURN );
			CommandMessage := cmTitle;
		end;
		cmTitle       :
		begin
			MainProcTitle;
		end;
		cmStartGame   :
		begin
			Objects[0] := TPlane.Create( 310, 240 );
			for i := 1 to Pred( MAX_OBJECT ) do begin
				if Assigned( Objects[i] ) then begin
					Objects[i].Free;
					Objects[i] := nil;
				end;
			end;

			StartTick      := timeGetTime( );
			CommandMessage := cmGame;
		end;
		cmGame        :
		begin
			MainProcGame;
		end;
		cmStartEnding :
		begin
			StartTick := timeGetTime( ) - StartTick;
			CommandMessage := cmEnding;
		end;
		cmEnding      :
		begin
			MainProcEnding;
		end;
	end;

	Result := ( HiByte( GetAsyncKeyState( VK_ESCAPE ) ) = 0 );
end;

function    TMainSystem.MainProcTitle  : Boolean;

	function  _( _1, _2, _3 : Real ) : TColorRef;
	var
		T          : Real;
		Rv, Gv, Bv : Real;
	begin
		T  := _1;
		Rv := 1 + _2 * Sin( T - 2 * PI / 3 );
		Gv := 1 + _2 * Sin( T );
		Bv := 1 + _2 * Sin( T + 2 * PI / 3 );
		T  := 255.9999 * _3 / 2;
		Result := RGB( Trunc( Rv * T ), Trunc( Gv * T ), Trunc( Bv * T ) );
	end;

const
	MAX_COLOR = 256;
	TimeCount : Integer = 0;
var
	OldFont : HFONT;
	DestDC  : HDC;
	s       : String;
	Size    : TSIZE;
	Factor  : Integer;
begin
	DDClearDevice( SmDirect3D.DirectDrawSurface, $0000 );

	SmDirect3D.DirectDrawSurface.GetDC( DestDC );
	try
		OldFont := SelectObject( DestDC, hFontTitle );

		Inc( TimeCount );
		Factor := ( ( TimeCount * 1 ) mod MAX_COLOR );

		SetTextColor( DestDC, _( 2*PI * Factor / MAX_COLOR, 1.0, 1.0 ) );
		SetBkMode( DestDC, TRANSPARENT );

		s := ' 2002';
		GetTextExtentPoint( DestDC, PChar( s ), Length( s ), Size );
		Textout( DestDC, ( GameOption.Width - Size.cx ) div 2, 50, PChar( s ), Length( s ) );

		SelectObject( DestDC, hFontNormal );
		SetTextColor( DestDC, RGB( 255, 240, 222 ) );

		s := ' :   , ָ  !';
		GetTextExtentPoint( DestDC, PChar( s ), Length( s ), Size );
		Textout( DestDC, ( GameOption.Width - Size.cx ) div 2, 115, PChar( s ), Length( s ) );

		SetTextColor( DestDC, RGB( 150, 150, 150 ) );

		s := '  2002';
		Textout( DestDC, 50, 175, PChar( s ), Length( s ) );
		s := '· ܰ  ħϰ ȴ.';
		Textout( DestDC, 50, 200, PChar( s ), Length( s ) );
		s := '   ΰ...';
		Textout( DestDC, 50, 225, PChar( s ), Length( s ) );
		s := '״  Ʒ忡  ٰ Ź';
		Textout( DestDC, 50, 250, PChar( s ), Length( s ) );
		s := 'ڶ  !  !  !!';
		Textout( DestDC, 50, 275, PChar( s ), Length( s ) );
		s := '   7 ״  ';
		Textout( DestDC, 50, 300, PChar( s ), Length( s ) );
		s := '  ''ź X'' ư  پ.';
		Textout( DestDC, 50, 325, PChar( s ), Length( s ) );
		s := ' źȯӿ    ߷½ǵ';
		Textout( DestDC, 50, 350, PChar( s ), Length( s ) );
		s := 'װ   ''̽ ''̾.......';
		Textout( DestDC, 50, 375, PChar( s ), Length( s ) );

		SetTextColor( DestDC, RGB( 255, 20, 50 ) );
		s := '< ENTER   >';
		Textout( DestDC, 400, 420, PChar( s ), Length( s ) );

	finally
		if ( OldFont <> 0 ) then
			SelectObject( DestDC, OldFont );
		SmDirect3D.DirectDrawSurface.ReleaseDC( DestDC );

	end;

	SmDirect3D.FlipPage;

	if LoByte( GetAsyncKeyState( VK_RETURN ) ) > 0 then
		CommandMessage := cmStartGame;

	Result := TRUE;
end;

function    TMainSystem.MainProcGame   : Boolean;
var
	Index     : Integer;
	MaxObject : Integer;
	DestDC    : HDC;
begin
	DDClearDevice( SmDirect3D.DirectDrawSurface, $0000 );

	MaxObject := ( timeGetTime( ) - StartTick ) div 100;
	if MaxObject < 0 then
		MaxObject := 0;
	if MaxObject > Pred( MAX_OBJECT ) then
		MaxObject := Pred( MAX_OBJECT );

	for Index := 0 to MaxObject do begin
		if Assigned( Objects[Index] ) then begin
			if not Objects[Index].MainProc( ) then begin
				Objects[Index].Free;
				Objects[Index] := nil;
			end;
		end
		else if Index > 0 then begin
			Objects[Index] := TParticle.Create( 10, 10, (Random(41)-20)*0.1, (Random(41)-20)*0.1, 0, 0 );
		end;
	end;

	if Assigned( Objects[0] ) then begin
		SmDirect3D.DirectDrawSurface.GetDC( DestDC );
		try
			Rectangle( DestDC, 10, 460, 10 + (Objects[0] as TPlane).RemainShield div 5, 475 );
		finally
			SmDirect3D.DirectDrawSurface.ReleaseDC( DestDC );

		end;
	end
	else begin
		CommandMessage := cmStartEnding;
	end;

	SmDirect3D.FlipPage;

	Result := TRUE;
end;

function    TMainSystem.MainProcEnding : Boolean;
var
	OldFont : HFONT;
	DestDC  : HDC;
	s       : String;
	Size    : TSIZE;
begin
	DDClearDevice( SmDirect3D.DirectDrawSurface, $0000 );

	SmDirect3D.DirectDrawSurface.GetDC( DestDC );
	try
		SetTextColor( DestDC, RGB( 240, 255, 250 ) );
		SetBkMode( DestDC, TRANSPARENT );

		if StartTick < 10000 then begin
			OldFont := SelectObject( DestDC, hFontNormal );
			s := 'ȸ. ƴü  .';
			GetTextExtentPoint( DestDC, PChar( s ), Length( s ), Size );
			Textout( DestDC, ( GameOption.Width - Size.cx ) div 2, 175, PChar( s ), Length( s ) );
			s := '10ʵ     Ű ?';
			GetTextExtentPoint( DestDC, PChar( s ), Length( s ), Size );
			Textout( DestDC, ( GameOption.Width - Size.cx ) div 2, 250, PChar( s ), Length( s ) );
		end else begin
			OldFont := SelectObject( DestDC, hFontNormal );
			s := '  !! ¦¦¦.';
			GetTextExtentPoint( DestDC, PChar( s ), Length( s ), Size );
			Textout( DestDC, ( GameOption.Width - Size.cx ) div 2, 175, PChar( s ), Length( s ) );
			Str( StartTick / 1000 : 6: 3, s );
			s := ' ð : ' + s + ' ';
			GetTextExtentPoint( DestDC, PChar( s ), Length( s ), Size );
			Textout( DestDC, ( GameOption.Width - Size.cx ) div 2, 250, PChar( s ), Length( s ) );
		end;

	finally
		if ( OldFont <> 0 ) then
			SelectObject( DestDC, OldFont );
		SmDirect3D.DirectDrawSurface.ReleaseDC( DestDC );

	end;

	SmDirect3D.FlipPage;

	if LoByte( GetAsyncKeyState( VK_RETURN ) ) > 0 then
		CommandMessage := cmStartTitle;

	Result := TRUE;
end;

end.

ϼп  ҽ !!!

