unit USmDx3D;

interface

uses
	Windows, USmUtil,
	DirectDraw, DDVer;

const
	TEXT_WIDTH  = 24;
	TEXT_HEIGHT = 24;
	TEXT_BIT    = 4;
	TEXT_BPL    = ( TEXT_WIDTH * TEXT_BIT + 7 ) div 8;

type
	TSpriteMode  = ( smCentered, smLeftTop, smLeftBottom );

type

	TSmDirect3D = class
		constructor Create( Handle : THandle; const DirectDraw : II_DirectDraw; const DirectDrawSurface : II_DirectDrawSurface );
		constructor Create3D( Handle : THandle; Width, Height : Integer; Depth : Integer; IsExclusive : Boolean );
		procedure   Free;

	private
		F3DOnly             : Boolean;
		FIsInitialized      : Boolean;
		FHandle             : THandle;
		FIsExclusive        : Boolean;
		FClipper            : IDirectDrawClipper;

		function  Initialze : Boolean;
		function  Finalize  : Boolean;

		function  RestoreSurfaces : HResult;

		procedure ModifyEdge ( var x, y : Integer; var ARect : TRect; Mode : TSpriteMode);

	public
		DirectDraw         : II_DirectDraw;
		PrimarySurface     : II_DirectDrawSurface;
		DirectDrawSurface  : II_DirectDrawSurface;

		procedure PutSprite  ( x, y : Integer; SourImage : II_DirectDrawSurface; ARect : TRect; Mode : TSpriteMode );
		procedure PutTile    ( x, y : Integer; SourImage : II_DirectDrawSurface; ARect : TRect; Mode : TSpriteMode );

		function  MakeItSo   ( DDResult : HResult ) : Boolean;
		procedure FlipPage;

	end;

implementation

uses
	UMainSys;

procedure SmD3DErrorOut( HRet : HRESULT; FuncName : string);
var
	OutString : string;
begin
	OutString := FuncName + ': ' + #13 + DDErrorString( HRet );
	MessageBox( 0, PChar(OutString), nil, MB_OK or MB_ICONSTOP );
end;

constructor TSmDirect3D.Create( Handle : THandle; const DirectDraw : II_DirectDraw; const DirectDrawSurface : II_DirectDrawSurface );
begin
	Inherited Create;

	F3DOnly                := FALSE;

	FHandle                := Handle;
	Self.DirectDraw        := DirectDraw;
	Self.PrimarySurface    := nil;
	Self.DirectDrawSurface := DirectDrawSurface;
	FClipper               := nil;

	FIsInitialized         := Initialze;
end;

constructor TSmDirect3D.Create3D( Handle : THandle; Width, Height : Integer; Depth : Integer; IsExclusive : Boolean );
var
	DDSurfaceDesc : TT_DDSurfaceDesc;
	ddsCaps       : TT_DDSCaps;
begin
	Inherited Create;

	FIsInitialized     := FALSE;

	F3DOnly            := TRUE;
	FHandle            := Handle;
	FIsExclusive       := IsExclusive;

	DirectDrawCreateEx( nil, DirectDraw, II_DirectDraw, nil );

	if not Assigned( DirectDraw ) then
		Exit;

	if ( IsExclusive ) then begin
		if ( DirectDraw.SetCooperativeLevel( FHandle, DDSCL_EXCLUSIVE or DDSCL_FULLSCREEN or DDSCL_ALLOWREBOOT ) <> DD_OK ) then
			Exit;

		if ( DirectDraw.SetDisplayMode( Width, Height, Depth, 0, 0 ) <> DD_OK ) then
			Exit;

		FillChar( DDSurfaceDesc, sizeof( DDSurfaceDesc ), 0 );

		DDSurfaceDesc.dwSize            := sizeof( DDSurfaceDesc );
		DDSurfaceDesc.dwFlags           := DDSD_CAPS or DDSD_BACKBUFFERCOUNT;
		DDSurfaceDesc.ddsCaps.dwCaps    := DDSCAPS_COMPLEX or DDSCAPS_FLIP or DDSCAPS_PRIMARYSURFACE;
		DDSurfaceDesc.dwBackBufferCount := 1;

		if ( DirectDraw.CreateSurface( DDSurfaceDesc, PrimarySurface, nil ) <> DD_OK ) then
			Exit;

		FillChar( ddsCaps, sizeof( ddsCaps ), 0 );
		ddsCaps.dwCaps := DDSCAPS_BACKBUFFER;
		if ( PrimarySurface.GetAttachedSurface( ddsCaps, DirectDrawSurface ) <> DD_OK ) then
			Exit;
	end
	else begin
		if ( DirectDraw.SetCooperativeLevel( FHandle, DDSCL_NORMAL ) <> DD_OK ) then
			Exit;

		FillChar( DDSurfaceDesc, sizeof( DDSurfaceDesc ), 0 );

		DDSurfaceDesc.dwSize            := sizeof( DDSurfaceDesc );
		DDSurfaceDesc.dwFlags           := DDSD_CAPS;
		DDSurfaceDesc.ddsCaps.dwCaps    := DDSCAPS_PRIMARYSURFACE;

		if ( DirectDraw.CreateSurface( DDSurfaceDesc, PrimarySurface, nil ) <> DD_OK ) then
			Exit;

		FillChar( DDSurfaceDesc, sizeof( DDSurfaceDesc ), 0 );

		DDSurfaceDesc.dwSize            := sizeof(DDSurfaceDesc);
		DDSurfaceDesc.dwFlags           := DDSD_CAPS or DDSD_HEIGHT or DDSD_WIDTH;
		DDSurfaceDesc.ddsCaps.dwCaps    := DDSCAPS_OFFSCREENPLAIN;
		DDSurfaceDesc.dwWidth           := Width;
		DDSurfaceDesc.dwHeight          := Height;

		if ( DirectDraw.CreateSurface( DDSurfaceDesc, DirectDrawSurface, nil ) <> DD_OK ) then
			Exit;

		if ( DirectDraw.CreateClipper( 0, FClipper, nil ) <> DD_OK ) then
			Exit;

		if ( FClipper.SetHWnd( 0, FHandle ) <> DD_OK ) then
			Exit;

		if ( PrimarySurface.SetClipper( FClipper ) <> DD_OK ) then
			Exit;
(*
		try
			BackBuffer.GetDC(DC);
			DirectDrawDesc.BPP := GetDeviceCaps(DC, BITSPIXEL) div 8;
		finally
			BackBuffer.ReleaseDC(DC);
		end;
*)
	end;

	FIsInitialized     := Initialze;
end;

procedure   TSmDirect3D.Free;
begin
	Finalize;

	if ( F3DOnly ) then begin
		if Assigned( FClipper ) then
			FClipper := nil;

		if ( FIsExclusive ) then begin
			if Assigned( PrimarySurface ) then
				PrimarySurface := nil;
			Pointer( DirectDrawSurface ) := nil;
		end
		else begin
			if Assigned( DirectDrawSurface ) then
				DirectDrawSurface := nil;
			if Assigned( PrimarySurface ) then
				PrimarySurface := nil;
		end;
	end;

	Inherited;
end;

function    TSmDirect3D.Initialze : Boolean;
begin
	Initialze := TRUE;
end;

function    TSmDirect3D.Finalize  : Boolean;
begin
	Finalize := TRUE;
end;

procedure TSmDirect3D.FlipPage;
var
	Rect : TRect;
begin
	if ( FIsExclusive ) then begin
		PrimarySurface.Flip( nil, DDFLIP_WAIT );
	end
	else begin
		Rect.TopLeft := Point( 0, 0 );
		ClientToScreen( FHandle, Rect.TopLeft );
		Rect := Bounds( Rect.Left, Rect.Top, 640, 480 );
		PrimarySurface.Blt( @Rect, DirectDrawSurface, PRect( nil ), DDFLIP_WAIT, PDDBltFX( nil ) );
	end;
end;

function    TSmDirect3D.MakeItSo( DDResult : HResult ) : Boolean;
begin
	// ȭ   Լ
	case DDResult of
		DD_OK             : Result := TRUE;
		DDERR_SURFACELOST : Result := RestoreSurfaces <> DD_OK;
		else                Result := DDResult <> DDERR_WASSTILLDRAWING;
	end;
end;

function    TSmDirect3D.RestoreSurfaces : HResult;
begin
	// ȭ Ҿ MakeItSo ȣ
	Result := PrimarySurface._Restore;
	if Result = DD_OK then begin
{
var
	i : integer;
		for i := 1 to DirectDrawDesc.MaxImage do begin
			Result := Image[i]._Restore;
			if Result <> DD_OK then
				Exit;
		end;
		if Result = DD_OK then
			DrawSurfaces;
}
	end;
end;

procedure   TSmDirect3D.ModifyEdge( var x, y : Integer; var ARect : TRect; Mode : TSpriteMode );
begin
	if Mode = smCentered then begin
		x := x - Succ( ARect.Right - ARect.Left ) div 2;
		y := y - ( ARect.Bottom - ARect.Top );
	end;

	if x < 0 then begin
		ARect.Left  := ARect.Left - x;
		x           := 0;
	end;
	if y < 0 then begin
		ARect.Top   := ARect.Top  - y;
		y           := 0;
	end;
	if ( x + ARect.Right - ARect.Left ) > Pred( GameOption.Width  ) then begin
		ARect.Right := GameOption.Width + ARect.Left - x;
	end;
	if ( y + ARect.Bottom - ARect.Top ) > Pred( GameOption.Height ) then begin
		ARect.Bottom := GameOption.Height + ARect.Top - y;
	end;
end;

procedure   TSmDirect3D.PutTile( x, y : Integer; SourImage : II_DirectDrawSurface; ARect : TRect; Mode : TSpriteMode );
begin
{
	Inc( x, DEFAULT_X_POS );
	Inc( y, DEFAULT_Y_POS );
}
	ModifyEdge( x, y, ARect, Mode );

	repeat
	until MakeItSo( DirectDrawSurface.BltFast( x, y, SourImage, @ARect, DDBLTFAST_NOCOLORKEY ) );
end;

procedure   TSmDirect3D.PutSprite( x, y : Integer; SourImage : II_DirectDrawSurface; ARect : TRect; Mode : TSpriteMode );
begin
{
	Inc( x, DEFAULT_X_POS );
	Inc( y, DEFAULT_Y_POS );
}
	ModifyEdge( x, y, ARect, Mode) ;
	repeat
	until MakeItSo( DirectDrawSurface.BltFast( x, y, SourImage, @ARect, DDBLTFAST_SRCCOLORKEY ) );
end;

end.
